@@ -2,7 +2,7 @@
  * ArithmeticError.java
  *
  * Copyright (C) 2003-2005 Peter Graves
- * $Id: ArithmeticError.java 14466 2013-04-24 12:50:40Z rschlatte $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -72,7 +72,7 @@ public class ArithmeticError extends LispError
     public ArithmeticError(String message)
     {
         super(StandardClass.ARITHMETIC_ERROR);
-        setFormatControl(message);
+        setFormatControl(message.replaceAll("~","~~"));
         setFormatArguments(NIL);
         setOperation(NIL);
         setOperands(NIL);
